'use strict';

class ContextMenu extends View {
    constructor(params) {
        super(params);

        this.element = _.ul({class: 'context-menu dropdown-menu', role: 'menu' });
        
        let existingMenu = _.find('.context-menu');
        
        if(typeof jQuery !== 'undefined') {
            if(existingMenu && existingMenu.length > 0) {
                this.element = existingMenu;
            }
        } else {
            if(existingMenu) {
                this.element = existingMenu;
            }
        }

        if(typeof jQuery !== 'undefined') {
            this.$element = this.element;
            this.element = this.$element[0];
        }

        this.element.innerHTML = '';

        this.element.style.position = 'absolute';
        this.element.style.zIndex = 1200;
        this.element.style.top = this.pos.y;
        this.element.style.left = this.pos.x;
        this.element.style.display = 'block';
        
        this.fetch();
    }
    
    render() {
        _.append(this.element,
            _.each(this.model, (label, func) => {
                if(func == '---') {
                    return _.li({class: 'dropdown-header'}, label);
                } else {
                    return _.li({class: typeof func === 'function' ? '' : 'disabled' },
                        _.a({tabindex: '-1', href: '#'},
                            label
                        ).click((e) => {
                            e.preventDefault();
                            e.stopPropagation();
                            
                            if(func) {
                                func(e);

                                this.remove();
                            }
                        })
                    );
                }
            })
        );

        _.append(_.find('body'), this.element);
    
        let rect = this.element.getBoundingClientRect();

        if(rect.left + rect.width > window.innerWidth) {
            this.element.style.left = (rect.left - rect.width) + 'px';

        } else if(rect.bottom > window.innerHeight) {
            this.element.style.top = (rect.top - rect.height) + 'px';
        }
    }
}

if(typeof jQuery !== 'undefined') {
    jQuery.fn.extend({
        exocontext: function(menuItems) {
            return this.each(function() {
                $(this).on('contextmenu', function(e) {
                    if(e.ctrlKey) {
                        return;
                    }

                    $('.context-menu-target-element').removeClass('context-menu-target-element');
                    
                    e.preventDefault();
                    e.stopPropagation();

                    if(e.which == 3) {
                        $(this).toggleClass('context-menu-target-element', true);
                        
                        var menu = new ContextMenu({
                            model: menuItems,
                            pos: {
                                x: e.pageX,
                                y: e.pageY
                            }
                        });
                    }
                });
            });
        }
    });

    // Event handling
    $('body').click(function(e) {
        if($(e.target).parents('.context-menu').length < 1) {
            $('.context-menu-target-element').removeClass('context-menu-target-element');
            ViewHelper.removeAll('ContextMenu');
        }
    });
}
